/* Ari's JAVA implementation of a labelled node specifically for the minsum algorithm. */

import java.awt.*;
import java.lang.String;

public class Node {
	public int left;          // left coordinate
	public int upper;         // upper coordinate
	public final int width;   // width of the node
	public final int height;  // height of the node
	public String name;       // the name of the node
	public int myType;        // type of node
	private int name_length;  // the length of name in screen bits
	private int name_height;  // the height of name in screen bits
	private Color myColor;    // the color of the node
	private boolean lit;      // is the node highlighted

	// for pretty displays
	public float forcex;
	public float forcey;

	// class constants
	public final static int unknownType=0; // an undesignated node
	public final static int symbolType=1;  // denotes a symbol node
	public final static int checkType=2;   // denotes a check node

	// sets up a new point
	public Node() { // a default node
		left=0;
		upper=0;
		width=0;
		height=0;
		name="";
		name_length=0;
		name_height=0;
		myColor=Color.black;     // the default color
		myType=unknownType;      // the default type
		lit=false;
	}

	public Node(int the_left,
							int the_upper,
							int the_width,
							int the_height)
	{
		left=the_left;
		upper=the_upper;
		width=the_width;
		height=the_height;
		name="";
		name_length=0;
		name_height=0;
		myColor=Color.black;     // the default color
		myType=unknownType;      // the default type
		lit=false;
	}

	public void setType(int theType)
		/* %M:  myType, myColor
			 %E:  sets the node to be of type theType; changes myColor according to the node type
			 */
	{
		myType=theType;

		switch (theType) {
		case Node.unknownType:
		case Node.symbolType:
			myColor=Color.black; break;
		case Node.checkType:
			myColor=Color.red;
		}
	}
	// sets the type of a node; this affects

	// sets the color of a node
 	public void setColor(Color theColor)
 	{
 		myColor=theColor;
 	}

	public void sethighLight(boolean light)
		/* %E: if light=true; sets node highlighting on else sets it off */
	{
		lit=light;
	}

	public void changeName(String the_name)
		/* %M: name
			 %E: changes the name (label) of the node
			 */
	{
		name=the_name;
		name_length=0; // reset name_length to be recalculated at the next "paint"
		name_height=0;
	}

	// paints the node on the Graphics device; should not change the painting mode
	public void paint(Graphics g) {
		//System.out.println("Drawing node...");
		Color currColor=g.getColor();
		g.setColor(myColor);
		if (name_length==0) // then recompute it
			name_length=g.getFontMetrics().stringWidth(name);
		if (name_height==0) // then recompute it
			name_height=g.getFontMetrics().getHeight();
		// label the node
		//System.out.println("name_length"+name_length);
		//System.out.println("name_height"+name_height);
		g.drawString(name,  // centers in the node
								 left+(width-name_length)/2,
								 upper+(height+name_height)/2);
		if (lit) {
			// highlight the node
			g.setXORMode(Color.white);
			g.fillOval(left,upper,width,height);
			g.setPaintMode();
		}
		else
			g.drawOval(left,upper,width,height);
		g.setColor(currColor);
	}
}
